@@ -2,7 +2,7 @@
 # PODNAME: App::Prove::Plugin::TestRail
 
 package App::Prove::Plugin::TestRail;
-$App::Prove::Plugin::TestRail::VERSION = '0.034';
+$App::Prove::Plugin::TestRail::VERSION = '0.035';
 use strict;
 use warnings;
 use utf8;
@@ -49,7 +49,6 @@ sub load {
     $ENV{'TESTRAIL_PLAN'}      = $params->{plan};
     $ENV{'TESTRAIL_CONFIGS'}   = $params->{configs};
     $ENV{'TESTRAIL_VERSION'}   = $params->{version};
-    $ENV{'TESTRAIL_CASEOK'}    = $params->{case_per_ok};
     $ENV{'TESTRAIL_STEPS'}     = $params->{step_results};
     $ENV{'TESTRAIL_SPAWN'}     = $params->{testsuite_id};
     $ENV{'TESTRAIL_TESTSUITE'} = $params->{testsuite};
@@ -73,7 +72,7 @@ App::Prove::Plugin::TestRail - Upload your TAP results to TestRail in realtime
 
 =head1 VERSION
 
-version 0.034
+version 0.035
 
 =head1 SYNOPSIS
 
@@ -96,7 +95,6 @@ If \$HOME/.testrailrc exists, it will be parsed for any of these values in a new
     plan=GosPlan
     configs=config1:config2:config3: ... :configN
     version=xx.xx.xx.xx
-    case_per_ok=0
     step_results=sr_sys_name
     lockname=internal_lock_name
     testsuite_id=123
